import { useState, useEffect } from "react";
import { Table, Button } from "react-bootstrap";
import { useTranslation } from "react-i18next";
import { faEdit, faTrashAlt } from "@fortawesome/free-regular-svg-icons";
import { faPlus } from "@fortawesome/free-solid-svg-icons";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import StaffModal from "../../components/StaffModal";
import { getBoatsFromApi, deleteBoat, addBoat, editBoat } from "../../api/boat";
import { getBoatTypesFromApi } from "../../api/boatTypes";
import { Boat, BoatType } from "../../types";
import validator from "validator";
import { Form } from "react-bootstrap";
import { Controller, useForm } from "react-hook-form";

function BoatManager() {
  const { t } = useTranslation();
  const [editElement, setEditElement] = useState<Boat | undefined>(undefined);
  const [deleteElement, setDeleteElement] = useState<number>(-1);
  const [isAdding, setAdding] = useState<boolean>(false);
  const [boats, setBoats] = useState<Boat[]>([]);
  const [boatTypes, setBoatTypes] = useState<BoatType[]>([]);
  const [loading, setLoading] = useState<boolean>(false);

  const {
    control,
    handleSubmit,
    setValue,
    formState: { errors, isValid },
  } = useForm<Boat>({
    mode: "onBlur",
    reValidateMode: 'onChange'
  });

  const getBoats = async () => {
    const boats = await getBoatsFromApi();
    setBoats(boats.result);
  };
  async function getBoatTypes() {
    setBoatTypes(await getBoatTypesFromApi());
  }

  useEffect(() => {
    getBoats();
    getBoatTypes();
  }, []);

  return (
    <div className="m-1 h-100">
      <Table responsive striped bordered hover>
        <thead>
          <tr>
            <th>{t("boatTypeOverview.Id")}</th>
            <th>{t("boatTypeOverview.Name")}</th>
            <th>{t("boatManager.Status")}</th>
            <th></th>
          </tr>
        </thead>
        <tbody style={{ overflowY: "scroll", maxHeight: "100px" }}>
          {boats.map((x, i) => (
            <tr key={x.id}>
              <td>{x.id}</td>
              <td>{x.name}</td>
              <td>{x.status}</td>
              <td>
                <div className="d-flex">
                  <div
                    className="mx-2"
                    onClick={() => {
                      setEditElement(x);
                    }}
                  >
                    <FontAwesomeIcon
                      icon={faEdit}
                      className="text-secondary clickableIcon"
                      size="1x"
                    />
                  </div>
                  <div
                    className="mx-2"
                    onClick={() => {
                      setDeleteElement(i);
                    }}
                  >
                    <FontAwesomeIcon
                      icon={faTrashAlt}
                      className="text-danger clickableIcon"
                      size="1x"
                    />
                  </div>
                </div>
              </td>
            </tr>
          ))}
        </tbody>
      </Table>
      <div
        className="d-flex px-2 py-1 justify-content-end bg-white border-top"
        style={{ position: "sticky", right: "5px", bottom: "0", zIndex: 3 }}
      >
        <Button
          onClick={() => {
            setAdding(true);
            setEditElement({ name: "" } as Boat);
            setValue('id', undefined as any);
            setValue('name', "");
          }}
          variant="secondary"
        >
          <FontAwesomeIcon icon={faPlus} className="text-white me-2" />
          {t("boatTypeOverview.addBoat")}
        </Button>
      </div>


      <StaffModal
        header={isAdding ? t("boatManager.AddBoat") : t("boatManager.EditBoat")}
        show={!!editElement}
        successText={isAdding ? t("common.Add") : t("common.Edit")}
        loadingNext={loading}
        disableNext={!isValid}
        onHide={() => {
          setEditElement(undefined);
          setAdding(false);
        }}
        onSuccess={() => {
          setLoading(true);
          handleSubmit((data) => {
            if (isAdding) {
              addBoat(data).then(async () => {
                await getBoats();
                setEditElement(undefined);
                setAdding(false);
                setLoading(false);
              });
            } else {
              editBoat(data).then(async () => {
                await getBoats();
                setEditElement(undefined);
                setAdding(false);
                setLoading(false);
              });
            }
          })()
        }}
      >
        <Form>
          {!isAdding && <Controller
            name="id"
            control={control}
            defaultValue={editElement?.id}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.Id")}</Form.Label>
                <Form.Control disabled type="text" {...field} />
              </div>
            )}
          />}
          <Controller
            name="name"
            control={control}
            defaultValue={editElement?.name}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.Name")}</Form.Label>
                <Form.Control type="text" {...field} isInvalid={!!errors.name} />
                <Form.Control.Feedback type="invalid">
                  {errors.name?.message}
                </Form.Control.Feedback>
              </div>
            )}
            rules={{
              required: {
                value: true,
                message: t("common.messages.required", {
                  val: t("boatManager.Name"),
                }),
              },
              validate: (value: string) =>
                validator.isAlphanumeric(value) ||
                t("common.messages.required").toString(),
            }}
          />
          <Controller
            name="boattype"
            control={control}
            defaultValue={editElement?.boattype}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.BoatType")}</Form.Label>
                <Form.Select aria-label="Default select example" {...field} isInvalid={!!errors.boattype}>
                  <option></option>
                  {boatTypes.map((x) => (
                    <option key={x.id} value={x.id}>{x.name}</option>
                  ))}
                </Form.Select>
                <Form.Control.Feedback type="invalid">
                  {errors.boattype?.message}
                </Form.Control.Feedback>
              </div>
            )}
            rules={{
              required: {
                value: true,
                message: t("common.messages.required", {
                  val: t("boatManager.BoatType"),
                }),
              },
            }}
          />
          <Controller
            name="status"
            control={control}
            defaultValue={editElement?.status}
            render={({ field }) => (
              <div className="mb-2">
                <Form.Label>{t("boatManager.Status")}</Form.Label>
                <Form.Select aria-label="Default select example" {...field} isInvalid={!!errors.status}>
                  <option value={1}>Default</option>
                  <option value={2}>Locked</option>
                </Form.Select>
                <Form.Control.Feedback type="invalid">
                  {errors.status?.message}
                </Form.Control.Feedback>
              </div>
            )}
            rules={{
              required: {
                value: true,
                message: t("common.messages.required", {
                  val: t("boatManager.Status"),
                }),
              },
            }}
          />
        </Form>

      </StaffModal>


      <StaffModal
        header={t("boatTypeOverview.DeleteBoatType")}
        hideColor="secondary"
        successText={t("common.Delete")}
        successColor="danger"
        show={deleteElement !== -1}
        loadingNext={loading}
        onHide={() => {
          setDeleteElement(-1);
        }}
        onSuccess={async () => {
          setLoading(true);
          await deleteBoat({ id: boats[deleteElement].id })
          await getBoats();
          setDeleteElement(-1);
          setLoading(false);
        }}
      >
        <span>
          {t("boatTypeOverview.messages.DeleteText", { val: boats[deleteElement]?.name })}
        </span>
      </StaffModal>
    </div>
  );
}

export default BoatManager;
